import type { App, Vue2 } from 'vue-demi';
export interface ModuleLoader<Props extends Record<string, any> = {}, Context = any> {
    /**
     * Alias of export setModuleLoaderOptions
     */
    setOptions: (options: ModuleLoaderOptions<Props>) => ModuleLoader;
    /**
     * Alias of export addErrorHandler
     */
    addErrorHandler: (handler: ErrorHandler) => ModuleLoader;
    /**
     * Alias of export removeErrorHandler
     */
    removeErrorHandler: (handler: ErrorHandler) => ModuleLoader;
    /**
     * Resolver
     */
    resolver: ReturnType<GetResolver<Context>> & {
        context: Context;
    };
    /**
     * Install ModuleLoader plugin
     */
    install: (app: App) => void;
}
/**
 * Register options from submodule bootstrap return
 */
export interface RegisterProperties {
    [key: string]: any;
}
/**
 * Main module loader typings
 */
export declare type ModuleLoaderOptions<Props extends Record<string, any> = {}> = {
    /**
     * Use sync mode to load submodules
     * @default false
     */
    sync?: boolean;
    /**
     * Global properties to pass to submodules,
     * set it before registerSubModules, otherwise it will not apply to submodules
     */
    props?: Props | (() => Props);
    /**
     * Loading shown,
     * return a function to hide loading
     */
    loading?: (lifecycle: 'mount' | 'unmount') => void | (() => void);
    /**
     * Register properties from sub module mount(functional module) return,
     * unregister in unmount if return a function
     */
    register?: (properties: RegisterProperties) => void | (() => void);
};
/**
 * Remote module
 */
export declare type ModuleRemoteConfig<Props extends Record<string, any> = {}> = {
    /**
     * Module name（Then name of submodule export to global）
     */
    name: string;
    /**
     * Entry script src
     */
    entry: string;
    /**
     * Style(s)（using ExtractTextPlugin to compile a css file）
     */
    styles?: string | string[];
    /**
     * Conditions for activating submodules
     * - string: prefix to match then location.pathname
     * - function: custom match function
     * - array: multiple match conditions
     * - default: will always load in app start
     */
    activeRule?: string | ((location: Location) => boolean) | Array<string | ((location: Location) => boolean)>;
    /**
     * Props，pass to submodule "mount" and "unmount" lifecycle
     */
    props?: Props;
};
/**
 * Functional module,
 * executes when after "beforeLoad" and before "beforeMount" lifecycle
 * return properties to register same as Mount
 */
export declare type FunctionalModule = (App: App | typeof Vue2) => void | RegisterProperties | Promise<void | RegisterProperties>;
/**
 * Router Guard
 */
export interface Router {
    beforeEach(guard: (to: any, from: any, next: () => void) => any): unknown;
    afterEach(guard: (to: any, from: any) => any): unknown;
}
/**
 * Registrable module
 */
export declare type RegistrableModule = ModuleRemoteConfig | Record<string, string | Omit<ModuleRemoteConfig, 'name'>> | FunctionalModule;
/**
 * Formated config from user registers
 */
export declare type InnerRegistrableModule = (Omit<ModuleRemoteConfig, 'styles'> & {
    styles?: string[];
}) | FunctionalModule;
/**
 * Lifecycle function
 */
export declare type Lifecycle = (module: InnerRegistrableModule) => void;
/**
 * Lifecycles
 */
export declare type Lifecycles = {
    /**
     * before bootstrap lifecycle
     */
    beforeLoad?: Lifecycle | Array<Lifecycle>;
    /**
     * before mount lifecycle
     */
    beforeMount?: Lifecycle | Array<Lifecycle>;
    /**
     * after mount lifecycle
     */
    afterMount?: Lifecycle | Array<Lifecycle>;
    /**
     * before unmount lifecycle
     */
    beforeUnmount?: Lifecycle | Array<Lifecycle>;
    /**
     * after unmount lifecycle
     */
    afterUnmount?: Lifecycle | Array<Lifecycle>;
};
/**
 * Error handler
 */
export declare type ErrorHandler = (error: Error, module: InnerRegistrableModule) => void;
/**
 * Loader resolver
 */
export interface GetResolver<Context = any> {
    /**
     * Get resolver
     * @param container container to append script, default is append to body in client side
     */
    (container?: string | ((proxy: Context) => Element)): {
        /**
         * execution context
         */
        context: Context;
        /**
         * execute script
         * @param entry remote script src
         */
        execScript<R = unknown>(entry: string): R | Promise<R>;
        /**
         * add styles
         * @param styles style href
         */
        addStyles(styles: string[]): void | Promise<void>;
        /**
         * remove styles
         * @param styles style href
         */
        removeStyles(styles: string[]): void | Promise<void>;
    };
}
